/******************************************************
*
*  Copyright:    Infinetix Incorporated, 2001
*
*  Program File: I2cDevice.cpp
*  Author:       Dan Parker
*
*  Description:  This file contains the Ci2cDev class
*		 member functions.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*******************************************************/

/* Standard "C" Includes                              */
#include <stdlib.h>
#include <sys/ioctl.h>
#include <stdio.h>

/* Driver Access Includes                             */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/* I2C Class Definition                               */
#include "i2cdevice.h"

/* i2c defines                                        */
#define TRUE                        1
#define FALSE                       0
#define I2C_CLASS_DEBUG             FALSE
#define RETRY_LIMIT                 4
#define IOCTL_SET_ADDRESS           0x0654
#define IOCTL_GET_REGISTER          0x0655
#define IOCTL_SET_REGISTER          0x0656


int Ci2cDev::i_I2cOpen = 0;
int Ci2cDev::i_I2cPtr = 0;

Ci2cDev::Ci2cDev( int BusAddr )
{
   if( I2C_CLASS_DEBUG )
      printf( "OPENING I2C BUS FOR A DEVICE\n" );
   i_I2cAddr = BusAddr;
   RetryCount = 0;
   OpenI2c();
}

Ci2cDev::~Ci2cDev( void )
{
   if( I2C_CLASS_DEBUG )
      printf( "CLOSING I2C BUS FOR A DEVICE\n" );
   CloseI2c();
}

void Ci2cDev::OpenI2c(void)
{
   if( i_I2cOpen == 0 )
   {
      if( I2C_CLASS_DEBUG )
         printf( "Calling open\n" );
      i_I2cPtr = open( "/dev/acb", O_RDWR );
      if( i_I2cPtr < 0 )
      {
         if( I2C_CLASS_DEBUG )
            printf( "ERROR - Could not open bus\n" );
      }
      else
         i_I2cOpen = 1;
   }
   else
      ++i_I2cOpen;
}

void Ci2cDev::CloseI2c(void)
{
   if( i_I2cOpen > 1 )
      --i_I2cOpen;
   else
      if( i_I2cOpen == 1 )
      {
         if( I2C_CLASS_DEBUG )
            printf( "Calling close\n" );
         if( close( i_I2cPtr ) < 0 )
         {
            if( I2C_CLASS_DEBUG )
               printf( "ERROR - could not close bus\n" );
         }
         else
            i_I2cOpen = 0;
      }
}

int Ci2cDev::I2cRead( int cmd, int bytes, unsigned char *data )
{
   int result = FALSE;
   int retry_cntr = 0;

   if( i_I2cOpen > 0 )
   {
      /* Set the devices address in the I2C bus register */
      ioctl( i_I2cPtr, IOCTL_SET_ADDRESS, i_I2cAddr );

      do
      {
         /* Write the command to the device */
         result = write( i_I2cPtr, (unsigned char *)(&cmd), 1 );
         if( I2C_CLASS_DEBUG )
            printf( "Ci2cDev::I2cRead write result: %i\n", result );

         if( result > 0 )
         {
            /* Read the data from the device */
            result = read( i_I2cPtr, data, bytes );
            if( I2C_CLASS_DEBUG )
               printf( "Ci2cDev::I2cRead read result: %i\n", result );
         }
         if( retry_cntr > 0 )
            ++RetryCount;
         ++retry_cntr;
      } while( (result < TRUE) && (retry_cntr < RETRY_LIMIT) );
   }
   else
      if( I2C_CLASS_DEBUG )
         printf( "ERROR - I2C bus not open\n" );

   return result;
}

int Ci2cDev::I2cWrite( unsigned char *data, int bytes )
{
   int result = FALSE;
   int retry_cntr = 0;
   int i;

   if( i_I2cOpen > 0 )
   {
      /* Set the devices address in the I2C bus register */
      ioctl( i_I2cPtr, IOCTL_SET_ADDRESS, i_I2cAddr );

      do
      {
         /* Write the command and data to the device */
         result = write( i_I2cPtr, data, bytes );
         if( retry_cntr > 0 )
            ++RetryCount;
         ++retry_cntr;
         if( I2C_CLASS_DEBUG )
         {
            printf( "Ci2cDev::I2cWrite write command result: %i\n", result );
            for( i = 0; i < bytes; ++i )
               printf( "%2X ", data[i] );
            printf( "\n" );
         }
      } while( (result < TRUE) && (retry_cntr < RETRY_LIMIT) );

   }
   else
      if( I2C_CLASS_DEBUG )
         printf( "ERROR - I2C bus not open\n" );

   return result;
}

int Ci2cDev::I2cGetChar( unsigned char *data )
{
   int result = FALSE;
   int retry_cntr = 0;

   if( i_I2cOpen > 0 )
   {
      /* Set the devices address in the I2C bus register */
      ioctl( i_I2cPtr, IOCTL_SET_ADDRESS, i_I2cAddr );

      do
      {
         /* Read the data from the device */
         result = read( i_I2cPtr, data, 1 );
         if( I2C_CLASS_DEBUG )
            printf( "Ci2cDev::I2cGetChar read result: %i\n", result );
         if( retry_cntr > 0 )
            ++RetryCount;
         ++retry_cntr;
      } while( (result < TRUE) && (retry_cntr < RETRY_LIMIT) );
   }
   else
      if( I2C_CLASS_DEBUG )
         printf( "ERROR - I2C bus not open\n" );

   return result;
}

int Ci2cDev::I2cPutChar( unsigned char data )
{
   int result = FALSE;
   int retry_cntr = 0;

   if( i_I2cOpen > 0 )
   {
      /* Set the devices address in the I2C bus register */
      ioctl( i_I2cPtr, IOCTL_SET_ADDRESS, i_I2cAddr );

      do
      {
         /* Write the command and data to the device */
         result = write( i_I2cPtr, &data, 1 );
         if( retry_cntr > 0 )
            ++RetryCount;
         ++retry_cntr;
         if( I2C_CLASS_DEBUG )
         {
            printf( "Ci2cDev::I2cPutChar write command result: %i\n", result );
            printf( "%2X\n ", data );
         }
      } while( (result < TRUE) && (retry_cntr < RETRY_LIMIT) );

   }
   else
      if( I2C_CLASS_DEBUG )
         printf( "ERROR - I2C bus not open\n" );

   return result;
}
